/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * Contributor(s):
 *   Jean-Bernard Marcon <jean-bernard.marcon@laposte.net>
 *   Alex Sirota 
 *
 * ***** END LICENSE BLOCK ***** */

/*
   id's of elements in document are equal to corresponding preferences
 */

var bmhOptions = null;
var bmhLib = null;

function bmhGetNavigatorWindow() {
    return Components.classes["@mozilla.org/appshell/window-mediator;1"]       
	.getService( Components.interfaces.nsIWindowMediator )
	.getMostRecentWindow( "navigator:browser" );
}

function bmhOptionsOnload() {
    try {
	bmhLib = bmhGetNavigatorWindow().bmhLib;
	if( bmhOptions == null ) {
	    bmhOptions = new BmhOptions();
	}
	else {
	    bmhOptions.init( bmhLib.getCurContentURL().toLowerCase() );
	}
	bmhOptions.checkUpload();
    }
    catch( e ) {
	try {
	    var sbs = Components.classes["@mozilla.org/intl/stringbundle;1"]
		.getService( Components.interfaces.nsIStringBundleService );
	    var strBundle = sbs.createBundle( "chrome://bookmarkshome/locale/bmh.properties" );
	    failStr = strBundle.GetStringFromName( "BmhFailureString" );
	    actionStr = strBundle.GetStringFromName( "OptionsFailureActionString" );
	    alert( failStr + actionStr );
	}
	catch( e1 ) {
	    alert( e )
	}
	throw( e );
    }
}

function bmhCompareCheckboxes( ch1, ch2 ) {
    if( ch1.value < ch2.value ) {
	return -1;
    }
    if( ch1.value > ch2.value ) {
	return 1;
    }
    return 0;
}

function BmhUpload( url ) {
    try {
	this.connected = false;
	// streams for commands and responses
	this.transportService = Components.classes["@mozilla.org/network/socket-transport-service;1"]
	    .getService( Components.interfaces.nsISocketTransportService );
	var host = bmhOptions.hostField.value.replace( /^ftp:\/\//, "" );
	this.commandTransport = this.transportService.createTransport( null, 0, host, 21, null );
	this.commandStream = this.commandTransport.openOutputStream( 0, 0, 0 );
	this.responseStream = this.commandTransport.openInputStream( 0, 0, 0 );
	this.scriptableResponseStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
	    .createInstance( Components.interfaces.nsIScriptableInputStream );
	this.scriptableResponseStream.init( this.responseStream );
	// commands, first is dummy
	this.commandQueue = [{ cmd: "start", param: "" }];
	this.commandQueue.push( { cmd: "USER", param: bmhOptions.userField.value } );
	this.commandQueue.push( { cmd: "PASS", param: bmhOptions.passwordField.value } );
	this.commandQueue.push( { cmd: "TYPE", param: "I" } );
	this.commandQueue.push( { cmd: "PASV", param: "" } );
	var remotePath = bmhOptions.pathField.value.replace( /\//g, "" );
	var remoteName = bmhOptions.fileField.value.replace( /\//g, "" );
	var remote =  "/" + remotePath + ( remotePath && remoteName ? ( "/" + remoteName ) : remoteName );
	this.commandQueue.push( { cmd: "STOR", param: remote } );
	this.commandQueue.push( { cmd: "QUIT", param: "" } );
	// streams for data
	this.file = bmhLib.getBmhFile( bmhOptions.optionsURL );
	this.fileStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
	    .createInstance( Components.interfaces.nsIFileInputStream );
	this.fileStream.init( this.file, 0x01, 0644, 0 );
	this.fileStream.QueryInterface( Components.interfaces.nsISeekableStream );
	this.fileStream.seek( 0, 0 );
	this.dataTransport = null;
	this.dataStream = null;
	// start session 
	var pump = Components.classes["@mozilla.org/network/input-stream-pump;1"]
	    .createInstance( Components.interfaces.nsIInputStreamPump );
	pump.init( this.responseStream, -1, -1, 0, 0, false );
	pump.asyncRead( this, null );
    }
    catch( e ) {
	alert( e );
	this.quit();
    }
}

BmhUpload.prototype = {


quit: function() {
	  if( this.commandTransport ) {
	      this.commandTransport.close( "" );
	  }
	  if( this.scriptableResponseStream ) {
	      this.scriptableResponseStream.close();
	  }
	  if( this.dataTransport ) {
	      this.dataTransport.close( "" );
	  }
	  if( this.fileStream ) {
	      this.fileStream.close();
	  }
      },

onStartRequest: function( request, context ) {
		},
onStopRequest: function( request, context, status ) {
		   if( ! this.connected ) {
		       bmhOptions.ftpLog.value += bmhLib.getLocaleString( "FTPNotConnectString" ); 
		   }
		   this.quit();
	       },

onDataAvailable: function( request, context, inputStream, offset, count ) {
		     try {
			 this.connected = true;
			 var response = this.scriptableResponseStream.read( count );      
			 bmhOptions.ftpLog.value += response.replace( /\r/g, "" ) ;
			 var matchArray; 
			 if( this.commandQueue.length ) {
			     var prevCmd = this.commandQueue[0];
			     // get lastline, \r may not be present
			     var matchArray = response.match( /([^\n]*)\r{0,1}\n$/ ); 
			     var lastline = ( matchArray ? matchArray[1]+"\n" : "" );
			     // get first digit of return code
			     matchArray = lastline.match( /^(\d)\d\d / );
			     if( ! matchArray ) {
				 // not last line
				 return;
			     }
			     else {
				 var returnCode = parseInt( matchArray[1] );
			     }
			     if( returnCode < 4 ) {
				 if( prevCmd.cmd == "PASV" ) {
				     var matchArray = response.match( /((?:\d+,){3}\d+),(\d+),(\d+)/ );
				     if( matchArray.length ) {
					 var ip = matchArray[1].replace( /,/g, "." );
					 var port = 256 * ( matchArray[2] % 256 ) + ( matchArray[3] % 256 );
					 // make connection
					 this.dataTransport = this.transportService.createTransport( 
						 null, 0, ip, port, null );
					 this.dataStream = this.dataTransport.openOutputStream( 
						 this.dataTransport.OPEN_BLOCKING, 0, 0 ); 
				     }
				 }
				 else if( prevCmd.cmd == "STOR" ) {
				     if( returnCode == 1 ) {
					 // Data connection accepted, returnCode is 2 when transfer is complete
					 this.dataStream.writeFrom( this.fileStream, this.file.fileSize );
					 this.dataStream.close();
					 this.fileStream.close();
					 return;
				     }
				 }
				 // remove previous command from queue 
				 this.commandQueue.shift();
			     }
			     else {
				 // error
				 this.commandQueue = [{ cmd: "QUIT", param: "" }];
			     }
			     if( this.commandQueue.length ) {
				 var nextCmd = this.commandQueue[0];
				 var outputData = nextCmd.cmd + ( nextCmd.param ? ( " " + nextCmd.param ) : "" ) + "\r\n";
				 this.commandStream.write( outputData, outputData.length );        
				 if( nextCmd.cmd == "PASS" ) {
				     outputData = "PASS\n"; 
				 }
				 bmhOptions.ftpLog.value += ( "\t" + outputData.replace( /\r/, "" ));
				 return;
			     }
			 }
		     }
		     catch( e ) {
			 alert( e );
			 this.quit();
		     }
		 }
}

function BmhOptions() {
    // general
    this.pageMenu = document.getElementById( "pageMenu" );
    this.iconsCheckbox = document.getElementById( "hideIcons" );
    this.hideEnginesCheckbox = document.getElementById( "hideSearchEngines" );
    this.engineMenu = document.getElementById( "engineMenu" );
    this.externalCheckbox = document.getElementById( "externalEngines" );
    this.dfltButton = document.getElementById( "defaultEngineButton" );
    this.whereMenu = document.getElementById( "newWindowLinks" );
    this.tooltipsMenu = document.getElementById( "tooltipDisplay" );
    this.nrOfColumnsField = document.getElementById( "nrOfColumns" );
    this.maxNameLengthField = document.getElementById( "maxNameLength" );
    this.maxHeaderLinesField = document.getElementById( "maxHeaderLines" );
    this.openFolderInMenu = document.getElementById( "openFolderIn" );
    // style
    this.stylesMenu = document.getElementById( "styles" );
    this.stylesheetField = document.getElementById( "stylesheet" );
    this.stylesheetField.action = 1;
    this.onlineBox = document.getElementById( "onlineBox" );
    this.onlineBox.style.display = "none";
    this.onlineStylesMenu = document.getElementById( "onlineStyles" );
    this.onlineLabel = document.getElementById( "onlineCaption" ).label;
    // folders 
    this.excludeMenu = document.getElementById( "excludeIndex" );
    this.subfolderCheckbox = document.getElementById( "subfolderCheckbox" );
    this.sortButton = document.getElementById( "sortFoldersButton" );
    this.sortButton.setAttribute( "label", bmhLib.getLocaleString( "SortLabel" ));
    this.contentFrame = document.getElementById( "folderFrame" ).contentWindow;
    this.inOrExcludedFoldersBox = this.contentFrame.document.getElementById( "inOrExcludedFolders" );
    this.folders = new Array();
    this.sortedFolders = new Array();
    this.folderBoxes = new Array();
    // file
    this.accountsMenu = document.getElementById( "accountsMenu" );
    this.hostField = document.getElementById( "ftpHost" );
    this.userField = document.getElementById( "ftpUser" );
    this.passwordField = document.getElementById( "ftpPassword" );
    this.pathField = document.getElementById( "ftpPath" );
    this.fileField = document.getElementById( "ftpRemoteFile" );
    this.ftpLog = document.getElementById( "ftpLog" );
    // minimum values for integer text fields
    this.minValues = new Array();
    this.minValues["nrOfColumns"] = 1;
    this.minValues["maxNameLength"] = 1;
    this.minValues["maxHeaderLines"] = 1;
    this.initStyle();
    // create folder checkboxes
    this.parseBookmarks();
    this.sortedFolders.sort( bmhCompareCheckboxes );
    this.initFileTab();
    // initialize options
    this.init( bmhLib.getCurContentURL().toLowerCase() );
}


BmhOptions.prototype = {

init: function( url ) {
	  // general
	  this.optionsURL = this.initPageMenu( url );
	  // icons
	  this.initialHideIcons = bmhLib.getRDFPref( "hideIcons", this.optionsURL );
	  this.iconsCheckbox.checked = this.initialHideIcons;
	  // engines
	  this.initialHideEngines = bmhLib.getRDFPref( "hideSearchEngines", this.optionsURL );
	  this.hideEnginesCheckbox.checked = this.initialHideEngines;
	  // check if default engine hasn't been removed
	  this.checkDefaultEngine( bmhLib.getRDFPref( "engineURL", this.optionsURL ));
	  this.initialDfltEngine = this.dfltEngine;
	  this.initialExternal = bmhLib.getRDFPref( this.externalCheckbox.id, this.optionsURL );
	  this.externalCheckbox.checked = this.initialExternal; 
	  this.hideEngines( this.initialHideEngines );
	  this.hiddenEngines = bmhLib.getArrayFromRDFPref( "hiddenEngines", this.optionsURL );
	  // target
	  this.initialNewWinPref = bmhLib.getPref( "open_newwindow", "browser.link." );
	  if( bmhLib.getRDFPref( "newWindowLinks", this.optionsURL )) {
	      this.whereMenu.selectedIndex = this.initialNewWinPref - 1; 
	  }
	  else {
	      this.whereMenu.selectedIndex = 0;
	  }
	  // tooltips
	  this.initialTooltipsIndex = bmhLib.getRDFPref( "tooltipDisplay", this.optionsURL );
	  this.tooltipsMenu.selectedIndex = this.initialTooltipsIndex;
	  this.nrOfColumnsField.value = bmhLib.getRDFPref( "nrOfColumns", this.optionsURL );
	  this.maxNameLengthField.value = bmhLib.getRDFPref( "maxNameLength", this.optionsURL );
	  this.maxHeaderLinesField.value = bmhLib.getRDFPref( "maxHeaderLines", this.optionsURL );
	  this.initialOpenFolderIn = bmhLib.getRDFPref( "openFolderIn", this.optionsURL );
	  this.openFolderInMenu.selectedIndex = this.initialOpenFolderIn;
	  // style
	  this.setPrefStyle();
	  // folders
	  this.excludeMenu.selectedIndex = bmhLib.getRDFPref( "excludeIndex", this.optionsURL );
	  this.checkFolders();
	  this.drawFolders( this.folders );
	  // file
	  document.getElementById( "pageLabel" ).value = this.optionsURL;
	  this.setPrefAccount();
	  this.fileField.value = bmhLib.getRDFPref( this.fileField.id, this.optionsURL );
      },

checkUpload: function() {
		 if( bmhLib.upload ) {
		     bmhLib.upload = false;
		     document.getElementById( "bmhTabs" ).selectedIndex = 3;
		     new BmhUpload();
		 }
		 else {
		     document.getElementById( "bmhTabs" ).selectedIndex = 0;
		 }
	     },

upload: function() {
	    var field;
	    if( ! this.hostField.value ) {
		field = bmhLib.getLocaleString( "HostString" );
		alert( bmhLib.getLocaleString( "RequiredFieldsString", [field] ));
		return;
	    }
	    if( ! this.userField.value ) {
		field = bmhLib.getLocaleString( "UserString" );
		alert( bmhLib.getLocaleString( "RequiredFieldsString", [field] ));
		return;
	    }
	    if( ! this.passwordField.value ) {
		field = bmhLib.getLocaleString( "PasswordString" );
		alert( bmhLib.getLocaleString( "RequiredFieldsString", [field] ));
		return;
	    }
	    new BmhUpload();
	},

initPageMenu: function( url ) {
		  var label = "about:" + bmhLib.getWhat( url );
		  var selItems = this.pageMenu.getElementsByAttribute( "label", label );
		  if( selItems.length ) {
		      this.pageMenu.selectedItem = selItems[0];
		  }
		  else {
		      this.pageMenu.selectedIndex = 0;
		  }
		  return this.pageMenu.selectedItem.label;
	      },

initFileTab: function() {
		 try {
		     var accounts = bmhLib.RDF.GetResource( bmhLib.accountsNS + "accountList" );
		     bmhLib.RDFC.Init( bmhLib.prefsDS, accounts );
		     var accountsEnum = bmhLib.RDFC.GetElements();
		     var curNode, curName, accountRes, index = 0, itemValue; 
		     var host = { value: "" }, user = { value: "" }, password = { value: "" }
		     // fill menu with accounts 
		     while( accountsEnum.hasMoreElements() ) {
			 curNode = accountsEnum.getNext();
			 curName = bmhLib.getProperty( curNode, bmhLib.NameArc, bmhLib.prefsDS );
			 itemValue = this.getAccountValue( curNode ); 
			 this.accountsMenu.insertItemAt( index, curName, itemValue );
			 index++
		     }
		 }
		 catch( e ) {
		     alert( e );
		 }
	     },

getAccountValue: function( node ) {
		     var host = { value: "" }, user = { value: "" }, password = { value: "" }
		     curName = bmhLib.getProperty( node, bmhLib.NameArc, bmhLib.prefsDS );
		     var accountRes = bmhLib.RDF.GetResource( bmhLib.accountsNS + curName );
		     host.value = bmhLib.getProperty( accountRes, bmhLib.hostArc , bmhLib.prefsDS ); 
		     var accountValue = "";
		     accountValue += encodeURI( host.value );
		     user.value = bmhLib.getProperty( accountRes, bmhLib.userArc, bmhLib.prefsDS ); 
		     accountValue += ( "|" + encodeURI( user.value )); 
		     var hostStr = host.value;
		     if( hostStr && user.value ) {
			 // for compatibility with fireftp, all hosts in pwm start with 'ftp.' 
			 try {
			     if( ! /^ftp\./.test( hostStr )) {
				 hostStr = "ftp." + host.value;
			     }
			     // if not found throws exception
			     Components.classes["@mozilla.org/passwordmanager;1"]
				 .getService( Components.interfaces.nsIPasswordManagerInternal )
				 .findPasswordEntry( host.value, user.value, null, host, user, password );   
			 }
			 catch( e ) {
			     // just fill in the password
			 }
		     }
		     accountValue += ( "|" + encodeURI( password.value ));
		     var path = bmhLib.getProperty( accountRes, bmhLib.pathArc, bmhLib.prefsDS );
		     accountValue += ( "|" + encodeURI( path )); 
		     return accountValue;
		 },

setPrefAccount: function() {
		    var prefAccountName = bmhLib.getRDFPref( "accountName", this.optionsURL );
		    var selItems = this.accountsMenu.getElementsByAttribute( "label", prefAccountName );
		    if( selItems.length ) {
			this.accountsMenu.selectedItem = selItems[0];
		    }
		    else if( this.accountsMenu.getElementsByTagName( "menuitem" ).length ) {
			this.accountsMenu.selectedIndex = 0;
		    }
		    this.changeAccount();
		},

changeAccount: function() {
		   this.hostField.value = ""; 
		   this.userField.value = ""; 
		   this.passwordField.value = ""; 
		   this.pathField.value = ""; 
		   if( this.accountsMenu.selectedItem ) {
		       var accountValues = this.accountsMenu.selectedItem.value.split( "|" );
		       if( accountValues.length == 4 ) {
			   this.hostField.value = decodeURI( accountValues[0] ); 
			   this.userField.value = decodeURI( accountValues[1] ); 
			   this.passwordField.value = decodeURI( accountValues[2] ); 
			   this.pathField.value = decodeURI( accountValues[3] ); 
		       }
		   }
	       },

updateAccount: function( field ) {
		   item = this.accountsMenu.selectedItem;
		   switch( field.id ) {
		       case "ftpHost":
			   item.value = item.value.replace( /^[^\|]*/, encodeURI( field.value ));
		       break;
		       case "ftpUser":
			   item.value = item.value.replace( /((?:[^\|]*\|){1})[^\|]*/, encodeURI( "$1"+field.value ));
		       break;
		       case "ftpPassword":
			   item.value = item.value.replace( /((?:[^\|]*\|){2})[^\|]*/, encodeURI( "$1"+field.value ));
		       break;
		       case "ftpPath":
			   item.value = item.value.replace( /((?:[^\|]*\|){3})[^\|]*/, encodeURI( "$1"+field.value ));
		       break;
		   }
	       },


removeEngine: function() {
		  const selectedItem = this.engineMenu.selectedItem;
		  if( ! selectedItem ) {
		      return;
		  }
		  const engineValue = selectedItem.getAttribute( "value" ); 
		  const engineName = selectedItem.getAttribute( "label" );
		  var enginePath = engineValue;
		  var imgPath = selectedItem.getAttribute( "src" ); 
		  // remove 'engine://' and unescape path
		  enginePath = decodeURIComponent( enginePath.substring( 9 ));
		  try {
		      var handler = Components.classes["@mozilla.org/network/protocol;1?name=file"]
			  .createInstance( Components.interfaces.nsIFileProtocolHandler );
		      // convert to native path spec: 
		      imgPath = handler.getFileFromURLSpec( imgPath ).path;
		  }
		  catch( e ) {}
		  if( ! this.getPromptService().confirm( window, bmhLib.getLocaleString( "DeletePluginString" ), 
			      bmhLib.getLocaleString( "DeletePluginPrompt", [engineName, enginePath, imgPath] ))) {
		      return;
		  }
		  // delete .src and image file
		  if( enginePath != "" ) {
		      this.deleteFile( enginePath );
		  }
		  if( imgPath != "" ) {
		      this.deleteFile( imgPath );
		  }
		  if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
		      // we're still seeing bmh 
		      this.delEngineFromBmhPage( engineValue );
		  }
		  // remove engine from options window
		  this.delEngineFromMenu( selectedItem );
		  // remove engine from searchbar
		  this.delEngineFromSearchBar( engineValue );
		  // remove engine from internet search data source
		  this.delEngineFromDS( engineValue );
	      },

delEngineFromDS: function( engineValue ) {
		     try {
			 const delEngineResource = bmhLib.RDF.GetResource( engineValue );
			 bmhLib.ISDS.Unassert( bmhLib.rootEngine, bmhLib.childArc, delEngineResource ); 
		     }
		     catch( e ) {
			 // do nothing, engine will be removed from DS after restart
		     }
		 },


delEngineFromMenu: function( engineItem ) {
		       engineItem.parentNode.removeChild( engineItem );
		       if( this.dfltEngine ) {
			   this.checkDefaultEngine( this.dfltEngine.value );
		       }
		       else {
			   // create new default
			   this.checkDefaultEngine( "" );
		       }
		   },

delEngineFromSearchBar: function( engineValue ) {
			    try {
				var mainDoc = bmhLib.getNavigatorDocument();
				var searchBar = mainDoc.getElementById( "searchbar" );
				var barPopup = mainDoc.getAnonymousElementByAttribute ( 
					searchBar, "anonid", "searchbar-popup" );
				var selBarItem = searchBar.getAttribute( "searchengine" );
				var delBarItems = barPopup.getElementsByAttribute( "value", engineValue );
				if( delBarItems.length == 1 ) {
				    // to be removed engine found
				    if( selBarItem == engineValue ) {
					// to be removed engine is selected
					var barItems = barPopup.getElementsByTagName( "menuitem" );
					var nextBarItem = delBarItems[0].nextSibling;
					if( barItems.length >= 3 ) {
					    // at least one more engine. first has value "rdf:*"
					    // (addengine item appended on popup)
					    if( nextBarItem ) {
						searchBar.setAttribute( "src", nextBarItem.getAttribute( "src" ));
						searchBar.setAttribute( "searchengine", nextBarItem.getAttribute( "value" ));
					    }
					    else {
						// to be removed item is last item
						searchBar.setAttribute( "src", barItems[1].getAttribute( "src" ));
						searchBar.setAttribute( "searchengine", barItems[1].getAttribute( "value" ));
					    }
					}
					else {
					    // no more engines
					    searchBar.setAttribute( "src", null );
					    searchBar.setAttribute( "searchengine", null );
					}
				    }
				    // remove engine from menu
				    barPopup.removeChild( delBarItems[0] );
				}
			    }
			    catch( e ) {}
			},

delEngineFromBmhPage: function( engineValue ) {
			  var option = this.findEngineOption( engineValue );
			  if( option ) {
			      var engineSelect = option.parentNode;
			      engineSelect.removeChild( option );
			  }
		      },

findEngineOption: function( engineValue ) {
		      if( ! engineValue ) {
			  return null;
		      }
		      var htmlDoc = bmhLib.getCurContentDocument();
		      if( htmlDoc.location.href == this.optionsURL ) {
			  var engineSelect = htmlDoc.getElementById( "engineSelect" );
			  var options = engineSelect.getElementsByTagName( "option" );
			  for( var i = 0; i < options.length; i++ ) {
			      if( options[i].value.substr( 0, engineValue.length ) == engineValue ) {
				  return options[i]
			      }
			  }
		      }
		      return null;
		  },

checkDefaultEngine: function( value ) {
			if( value == "" ) {
			    // values at startup may be ""
			    value = "bmh";
			}
			// check if default engine hasn't been removed
			var temp = this.engineMenu.getElementsByAttribute( "value", value );
			if(( this.engineMenu.getElementsByTagName( "menuitem" ).length >= 2 )) {
			    // at least one engine present. first item has value "rdf:*"
			    if( temp.length == 0 ) {
				// default engine removed. set top engine as default
				this.engineMenu.selectedIndex = 1;

			    }
			    else {
				this.engineMenu.selectedItem = temp[0];
			    }
			    this.setDefaultEngine();
			}	  
			else {
			    // zero engines
			    this.dfltEngine = null;
			    this.dfltButton.setAttribute( "label", "" );
			    this.dfltButton.setAttribute( "image", "" );
			    this.engineMenu.selectedItem = null;
			    bmhLib.selectEngine( null );
			}
		    },

deleteFile: function ( fileName ) {
		try {
		    var file  = Components.classes["@mozilla.org/file/local;1"]
			.createInstance( Components.interfaces.nsILocalFile );
		    file.initWithPath( fileName );
		    file.remove( false );
		}
		catch( e ) {
		    alert( bmhLib.getLocaleString( "FileProblemString",[e.toString(),fileName] ));
		}
	    },

apply: function() {
	   try {
	       var prefs = new Array();
	       // general
	       prefs.push( this.parseValue( this.iconsCheckbox.id, this.iconsCheckbox.checked ));
	       prefs.push( this.parseValue( this.hideEnginesCheckbox.id, this.hideEnginesCheckbox.checked ));
	       prefs.push( this.parseValue( "engineURL", ( this.dfltEngine ? this.dfltEngine.value : "" )));
	       prefs.push( this.parseValue( this.externalCheckbox.id, this.externalCheckbox.checked ));
	       var hiddenEnginesString = bmhLib.getRDFPrefFromArray( this.hiddenEngines );
	       prefs.push( this.parseValue( "hiddenEngines", hiddenEnginesString ));
	       prefs.push( this.parseValue( this.whereMenu.id, ( this.whereMenu.selectedIndex > 0 )));
	       prefs.push( this.parseValue( this.tooltipsMenu.id, this.tooltipsMenu.selectedIndex ));
	       prefs.push( this.parseValue( this.nrOfColumnsField.id, this.nrOfColumnsField.value ));
	       prefs.push( this.parseValue( this.maxNameLengthField.id, this.maxNameLengthField.value ));
	       prefs.push( this.parseValue( this.maxHeaderLinesField.id, this.maxHeaderLinesField.value ));
	       prefs.push( this.parseValue( this.openFolderInMenu.id, this.openFolderInMenu.selectedIndex ));
	       // style
	       // apply unsaved changes to current sheet
	       if( this.stylesMenu.selectedItem.label == this.onlineLabel ) {
		   if( this.saveStylePrompt( this.onlineStylesMenu.selectedItem.label, this.stylesheetField.value )) {
		       // if not saved, keep previous default
		       prefs.push( this.parseValue( "styleName", this.getCurStyleName() ));
		   }
	       }
	       else {
		   this.changeSheetProperty( this.getCurStyleName(), this.stylesheetField.value );
		   prefs.push( this.parseValue( "styleName", this.getCurStyleName() ));
	       }
	       // folders
	       prefs.push( this.parseValue( this.excludeMenu.id, this.excludeMenu.selectedIndex ));
	       var sortedInOrExcludedFolders = new Array(); 
	       for( var i = 0; i < this.sortedFolders.length; i++ ) {
		   if( this.sortedFolders[i].checked ) {
		       sortedInOrExcludedFolders.push( this.sortedFolders[i].value.toString() );
		   }
	       }
	       var inOrExcludedString = bmhLib.getPrefFromArray( sortedInOrExcludedFolders );
	       prefs.push( this.parseValue( this.inOrExcludedFoldersBox.id, inOrExcludedString ));
	       // file
	       prefs.push( this.parseValue( this.fileField.id, this.fileField.value ));
	       // set prefs
	       // set dynamic here in case prefs changed but loop fails
	       bmhLib.setDynamicsPref( this.optionsURL, true );
	       for( i = 0; i < prefs.length; i++ ) {
		   bmhLib.setRDFPref( prefs[i].name, prefs[i].value, this.optionsURL );
	       }
	       // adapt initial values
	       this.initialHideEngines = this.hideEnginesCheckbox.checked;
	       this.initialExternal = this.externalCheckbox.checked;
	       this.initialNewWinPref = bmhLib.getPref( "open_newwindow", "browser.link." );
	       this.initialHideIcons = this.iconsCheckbox.checked;
	       this.initialTooltipsIndex = this.tooltipsMenu.selectedIndex;
	       this.initialDfltEngine = this.dfltEngine;
	       this.initialOpenFolderIn = this.openFolderInMenu.selectedIndex;
	       // write stylesfile
	       this.flushDataSources();
	       // reload if focused content matches url of options window
	       if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
		   bmhLib.reloadPage();
	       }
	   }
	   catch( e ) {
	       // improper value in field
	       alert( e );
	   }
       },

parseValue: function( aName, aValue ) {
		var aType = bmhLib.getPrefType( aName );
		var minVal = this.minValues[aName];
		var impropervalue = bmhLib.getLocaleString( "ImproperValueString" );
		if( minVal == null ) {
		    // value does not need to be checked
		    return { name: aName, type: aType, value: aValue };
		}
		aValue = parseInt( aValue );
		if( isNaN( aValue ) || aValue < minVal ) {
		    throw aName + " " + impropervalue;
		}
		return { name: aName, type: aType, value: aValue };
	    },

close: function() {
	   // reset page to last applied settings 
	   this.openWhere( this.initialNewWinPref - 1 );
	   this.hideEngines( this.initialHideEngines ); 
	   this.openExternally( this.initialExternal ); 
	   this.hideIcons( this.initialHideIcons ); 
	   this.engineMenu.selectedItem = this.initialDfltEngine;
	   this.setDefaultEngine();
	   this.openFoldersIn( this.initialOpenFolderIn );
	   // check all styles for changes
	   var stylesList = bmhLib.RDF.GetResource( bmhLib.stylesNS + "stylesList" );
	   bmhLib.RDFC.Init( bmhLib.stylesDS, stylesList );
	   var stylesEnum = bmhLib.RDFC.GetElements();
	   var promptService = this.getPromptService(); 
	   var flags = promptService.BUTTON_TITLE_DONT_SAVE * promptService.BUTTON_POS_0;
	   flags += promptService.BUTTON_TITLE_SAVE * promptService.BUTTON_POS_1;
	   var saveasstring = bmhLib.getLocaleString( "SaveChangesToString" );
	   var savechangesstring = bmhLib.getLocaleString( "SaveChangesString" );
	   var curNode, curName, oldSheet, newSheet; 
	   var message, index = 0;
	   var styleItems = this.stylesMenu.getElementsByTagName( "menuitem" );
	   while( stylesEnum.hasMoreElements() ) {
	       curNode = stylesEnum.getNext();
	       oldSheet = bmhLib.getProperty( curNode, bmhLib.sheetArc, bmhLib.stylesDS );
	       newSheet = styleItems[index].value;  
	       if( newSheet.replace( /\s*/g, "" ) !=  oldSheet.replace( /\s*/g, "" )) {
		   // sheet has changed
		   curName = styleItems[index].label; //this.stylesArray[index].name;
		   message = saveasstring + " " + curName + "?";
		   if( promptService.confirmEx(window, savechangesstring, 
			       message, flags, null, null, null, null, { value: 0 } )) {
		       this.changeSheetProperty( curName, newSheet );
		   }
	       }
	       index++;
	   }
	   // reset page to last applied style 
	   this.setPrefStyle();
	   // check accounts for changes
	   var accountList = bmhLib.RDF.GetResource( bmhLib.accountsNS + "accountList" );
	   bmhLib.RDFC.Init( bmhLib.prefsDS, accountList );
	   var accountsEnum = bmhLib.RDFC.GetElements();
	   var accountItems = this.accountsMenu.getElementsByTagName( "menuitem" );
	   var oldAccount, newAccount;
	   index = 0;
	   while( accountsEnum.hasMoreElements() ) {
	       curNode = accountsEnum.getNext();
	       oldAccount = this.getAccountValue( curNode );
	       newAccount = accountItems[index].value;  
	       if( oldAccount != newAccount ) {
		   // account has changed
		   curName = accountItems[index].label; 
		   message = saveasstring + " " + curName + "?";
		   if( promptService.confirmEx(window, savechangesstring, 
			       message, flags, null, null, null, null, { value: 0 } )) {
		       this.changeAccountProperties( curName );
		   }
	       }
	       index++;
	   }
	   this.flushDataSources();
       },

hideEngines: function( hide ) {
		 try {
		     document.getElementById( "engineBox" ).style.display = ( hide ? "none" : "" );
		     var doc = bmhLib.getCurContentDocument();
		     if( doc.location.href.toLowerCase() == this.optionsURL ) {
			 if( ! hide ) { 
			     // check if not all engines are hidden individually
			     var dfltOption = this.findEngineOption( this.dfltEngine.value );
			     bmhLib.selectEngine( dfltOption );
			 }
			 else {
			     doc.getElementById( "searchForm" ).style.display = "none";
			 }
		     }
		 }
		 catch( e ) {}
	     },

openExternally: function( external ) {
		    if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
			var searchForm = bmhLib.getCurContentDocument()
			    .getElementById( "searchForm" );
			searchForm.setAttribute( "class", ( external ? "blank" : "self" ));
		    }
		},

toggleEngineDisplay: function() {
			 try {
			     var selectedItem = this.engineMenu.selectedItem;
			     if( ! selectedItem ) {
				 return;
			     }
			     var option = this.findEngineOption( selectedItem.value );
			     var index = bmhLib.binarySearch( selectedItem.value, this.hiddenEngines );
			     if( index >= 0 ) {
				 // engine in hidden engines list, remove it and show option
				 this.hiddenEngines.splice( index, 1 );
				 if( option ) {
				     option.style.display = "";
				 }
			     }
			     else {
				 // engine not in list. Add and hide option
				 this.hiddenEngines.push( selectedItem.value );
				 this.hiddenEngines.sort();
				 if( option ) {
				     option.style.display = "none";
				 }
			     }
			     var dfltOption = this.findEngineOption( this.dfltEngine.value );
			     if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
				 bmhLib.selectEngine( dfltOption );
			     }
			 }
			 catch( e ) {}
		     },

setDefaultEngine: function() {
		      try {
			  var selectedItem = this.engineMenu.selectedItem;
			  if( ! selectedItem ) {
			      return;
			  }
			  this.dfltButton.setAttribute( "image", selectedItem.getAttribute( "src" ));
			  this.dfltButton.setAttribute( "label", selectedItem.getAttribute( "label" ));
			  this.dfltEngine = selectedItem;
			  if( ! this.hideEnginesCheckbox.checked && 
				  ( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL )) {
			      var dfltOption = this.findEngineOption( selectedItem.value );
			      bmhLib.selectEngine( dfltOption );
			  }
		      }
		      catch( e ) {}
		  },

openFoldersIn: function( index ) {
		   bmhLib.setRDFPref( this.openFolderInMenu.id, index, this.optionsURL );
	       },
		  
openWhere: function( where ) {
	       try {
		   var anchorLis = bmhLib.getCurContentDocument().getElementById( "theRow" )
		       .getElementsByTagName( "li" );
		   var i;
		   if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
		       if( where > 0 ) {
			   bmhLib.setPref( "open_newwindow", where + 1, "browser.link." );
			   for( i = 0; i < anchorLis.length; i++ ) {
			       anchor = anchorLis[i].firstChild;
			       if( anchor ) {
				   anchor.setAttribute( "target", "_blank" );
			       }
			   }
		       }
		       else {
			   for( i = 0; i < anchorLis.length; i++ ) {
			       anchor = anchorLis[i].firstChild;
			       if( anchor ) {
				   anchor.removeAttribute( "target" );
			       }
			   }
		       }

		   }
	       }
	       catch( e ) {}
	   },

hideIcons: function( hide ) {
	       try {
		   var i, icons;
		   if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
		       //icons = this.browser.contentDocument.getElementsByTagName( "img" );
		       icons = bmhLib.getCurContentDocument().getElementsByTagName( "img" );
		       for( i = 0; i < icons.length; i++ ) {
			   if( icons[i].getAttribute( "class" ) == "normalIcon" ) {
			       icons[i].style.display = ( hide ? "none ! important" : "inline ! important" );
			   }
		       }
		   }
	       }
	       catch( e ) {}
	   },

setTooltipDisplay: function( displayIndex ) {
		       try {
			   var i, divs;
			   if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
			       divs = bmhLib.getCurContentDocument().getElementsByTagName( "div" );
			       var prevSibling, hoverClass;
			       for( i = 0; i < divs.length; i++ ) {
				   if( divs[i].getAttribute( "class" ) == "hoverDiv" ) {
				       if( displayIndex == 0 ) {
					   divs[i].style.display = "";
				       }
				       else if( displayIndex == 2 ) {
					   divs[i].style.display = "none";
				       }
				       else {
					   prevSibling = divs[i].previousSibling;
					   if( prevSibling ) {
					       if( prevSibling.getAttribute( "class" ) == "liveAnchor" ) {
						   divs[i].style.display = "";
					       }
					       else {
						   divs[i].style.display = "none";
					       }
					   }
				       }
				   }
			       }
			   }
		       }
		       catch( e ) {}
		   },


initStyle: function() {
	       // RDF variables
	       var stylesList = bmhLib.RDF.GetResource( bmhLib.stylesNS + "stylesList" );
	       bmhLib.RDFC.Init( bmhLib.stylesDS, stylesList );
	       var stylesEnum = bmhLib.RDFC.GetElements();
	       var curNode, curName, curSheet; //, stylesMenuitem; 
	       var index = 0;
	       // fill menu with styles
	       while( stylesEnum.hasMoreElements() ) {
		   curNode = stylesEnum.getNext();
		   curName = bmhLib.getProperty( curNode, bmhLib.nameArc, bmhLib.stylesDS );
		   curSheet = bmhLib.getProperty( curNode, bmhLib.sheetArc, bmhLib.stylesDS );
		   this.stylesMenu.insertItemAt( index, curName, bmhLib.formatSheet( curSheet ));
		   index++;
	       }
	       // set color in colorpicker
	       var color = "#FF0000";
	       document.getElementsByTagName( "colorpicker" )[0].color = color;
	       this.showColor( color ); 
	   },

setPrefStyle: function() {
		  try {
		      var prefStyleName = bmhLib.getRDFPref( "styleName", this.optionsURL );
		      var items = this.stylesMenu.getElementsByAttribute( "label", prefStyleName );
		      if( items.length ) {
			  this.stylesMenu.selectedItem = items[0];
		      }
		      else {
			  this.stylesMenu.selectedIndex = 0;
		      }
		      this.changeStyle();
		  }
		  catch( e ) {}
	      },


flushDataSources: function() {
		      try {
			  bmhLib.stylesDS.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			      .Flush();
			  bmhLib.prefsDS.QueryInterface( Components.interfaces.nsIRDFRemoteDataSource )
			      .Flush();
		      }
		      catch( e ) {}
		  },

updateStyle: function( action ) {
		 try {
		     switch( action ) {
			 case 1:
			     // update style with timeout, set action prop to 2 
			     this.stylesheetField.action = 2;
			     window.setTimeout( "bmhOptions.updateStyle( 3 )", 200 );
			     break;
			     /*
				case 2:
			     // waiting for update, do nothing  
			     break;
			      */
			 case 3: 
			     // execute update, set action prop to 1
			     if( bmhLib.getCurContentURL().toLowerCase() == this.optionsURL ) {
				 var styleNode = bmhLib.getCurContentDocument().getElementsByTagName( "style" )[0];
				 styleNode.innerHTML = this.stylesheetField.value;
				 this.stylesMenu.selectedItem.value = this.stylesheetField.value;
				 this.stylesheetField.action = 1;
			     }
			     break;
		     }
		 }
		 catch( e ) {}
	     },

changeSheetProperty: function( name, sheetStr ) { 
			 // get resources for style
			 var styleResource = bmhLib.RDF.GetResource( bmhLib.stylesNS + name );
			 var oldSheet = bmhLib.stylesDS.GetTarget( styleResource, bmhLib.sheetArc, true );
			 var newSheet = bmhLib.RDF.GetLiteral( sheetStr );
			 // change styles datasource
			 bmhLib.stylesDS.Change( styleResource, bmhLib.sheetArc, oldSheet, newSheet );
			 var matches = this.stylesMenu.getElementsByAttribute( "label", name );
			 if( matches.length ) { 
			     this.stylesMenu.selectedItem = matches[0];
			 }
		     },

getPromptService: function() {
		      return Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
			  .getService( Components.interfaces.nsIPromptService );
		  },

changeAccountProperties: function( name ) {
			     var accountRes = bmhLib.RDF.GetResource( bmhLib.accountsNS + name );
			     var oldHost = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.hostArc, true );
			     var newHost = bmhLib.RDF.GetLiteral( this.hostField.value );
			     var oldUser = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.userArc, true );
			     var newUser = bmhLib.RDF.GetLiteral( this.userField.value );
			     var oldPath = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.pathArc, true );
			     var newPath = bmhLib.RDF.GetLiteral( this.pathField.value );
			     if( oldHost ) { 
				 bmhLib.prefsDS.Change( accountRes, bmhLib.hostArc, oldHost, newHost );
			     }
			     else {
				 bmhLib.prefsDS.Assert( accountRes, bmhLib.hostArc, newHost );
			     }
			     if( oldUser ) {
				 bmhLib.prefsDS.Change( accountRes, bmhLib.userArc, oldUser, newUser );
			     }
			     else {
				 bmhLib.prefsDS.Assert( accountRes, bmhLib.userArc, newUser );
			     }
			     if( this.passwordField.value && this.hostField.value && this.userField.value ) { 
				 try {
				     var hostStr = this.hostField.value;
				     if( /^ftp\./.test( hostStr )) {
					 hostStr = "ftp." + hostStr;
					 // if already there, no changes
					 Components.classes["@mozilla.org/passwordmanager;1"]
					     .getService( Components.interfaces.nsIPasswordManager )
					     .addUser( hostStr, this.userField.value, this.passwordField.value  );   
				     }
				 }
				 catch( e ) {
				     alert( e );
				 }
			     }
			     if( oldPath ) {
				 bmhLib.prefsDS.Change( accountRes, bmhLib.pathArc, oldPath, newPath );
			     }
			     else {
				 bmhLib.prefsDS.Change( accountRes, bmhLib.pathArc, oldPath, newPath );
			     }
			 },



saveAccountPrompt: function( item ) {
		       var promptService = this.getPromptService();
		       var name = { value: "" };
		       if( item ) {
			   name.value = item.label;
		       }
		       var saveAccountAsStr = bmhLib.getLocaleString( "SaveAccountAsString" );
		       var accountNameStr = bmhLib.getLocaleString( "AccountNameString" );
		       accountExistsStr = bmhLib.getLocaleString( "AccountExistsString" );
		       accountAlreadyStr = bmhLib.getLocaleString( "StyleAlreadyString" );
		       // prompt until user has given new name or wants to change the style 
		       // of existing name
		       while( promptService.prompt( window, 
				   saveAccountAsStr, accountNameStr, name, null, {value:0} )) {
			   if( ! name.value ) { 
			       continue;
			   }
			   if( this.accountsMenu.getElementsByAttribute( "label", name.value ).length ) {
			       // accountname exists
			       if( promptService.confirm( window, accountExistsStr, 
					   name.value + " " + accountAlreadyStr )) {
				   // change style belonging to name
				   this.changeAccountProperties( name.value );
				   return;
			       }
			   }
			   else {
			       this.saveAccountAs( name.value, item );
			       return;
			   }
		       }
		   },


saveStylePrompt: function( nameStr, sheet ) {
		     var promptService = this.getPromptService();
		     var name = { value: nameStr };
		     var saveStyleAsStr = bmhLib.getLocaleString( "SaveStyleAsString" );
		     var styleNameStr = bmhLib.getLocaleString( "StyleNameString" );
		     styleExistsStr = bmhLib.getLocaleString( "StyleExistsString" );
		     styleAlreadyStr = bmhLib.getLocaleString( "StyleAlreadyString" );
		     var styleItems = this.stylesMenu.getElementsByTagName( "menuitem" );
		     // prompt until user has given new name or wants to change the style 
		     // of existing name
		     while( promptService.prompt( window, saveStyleAsStr, styleNameStr, 
				 name, null, {value:0} )) {
			 if( ! name.value ) {
			     continue;
			 }
			 if( this.stylesMenu.getElementsByAttribute( "label", name.value ).length ) {
			     // stylename exists
			     if( promptService.confirm( window, styleExistsStr, name.value + " " + styleAlreadyStr )) {
				 // change style belonging to name
				 this.changeSheetProperty( name.value, sheet );
				 return true;
			     }
			 }
			 else {
			     this.saveStyleAs( name.value, sheet );
			     return true;
			 }
		     }
		     return false;
		 },

saveStyleAsPrompt: function() {
		       if( this.stylesMenu.selectedItem.label == this.onlineLabel ) {
			   this.saveStylePrompt( this.onlineStylesMenu.selectedItem.label, 
				   this.stylesheetField.value );
		       }
		       else {
			   this.saveStylePrompt( this.stylesMenu.selectedItem.label,
				   this.stylesheetField.value );
		       }
		   },

deleteAccountPrompt: function() {
			 if( ! this.accountsMenu.selectedItem ) {
			     return;
			 }
			 var deleteAccount = bmhLib.getLocaleString( "DeleteStyleString" );
			 var deleteconfirm = bmhLib.getLocaleString( "DeleteConfirmString" );
			 var name = this.accountsMenu.selectedItem.label;
			 var promptService = this.getPromptService();
			 if( promptService.confirm( window, "", 
				     deleteconfirm + " " + name + "?" )) {
			     this.deleteAccount();
			 }
		     },

deleteAccount: function() {
		   if( ! this.accountsMenu.selectedItem ) {
		       return;
		   }
		   var index = this.stylesMenu.selectedIndex;
		   var name = this.accountsMenu.selectedItem.label; 
		   var accountRes = bmhLib.RDF.GetResource( bmhLib.accountsNS + name );
		   var nameNode = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.NameArc, true );
		   var hostNode = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.hostArc, true );
		   var userNode = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.userArc, true );
		   var pathNode = bmhLib.prefsDS.GetTarget( accountRes, bmhLib.pathArc, true );
		   if( nameNode ) {
		       bmhLib.prefsDS.Unassert( accountRes, bmhLib.NameArc, nameNode );
		   }
		   if( hostNode ) {
		       bmhLib.prefsDS.Unassert( accountRes, bmhLib.hostArc, hostNode );
		   }
		   if( userNode ) {
		       bmhLib.prefsDS.Unassert( accountRes, bmhLib.userArc, userNode );
		   }
		   if( pathNode ) {
		       bmhLib.prefsDS.Unassert( accountRes, bmhLib.pathArc, pathNode );
		   }
		   try {
		       var hostStr = this.hostField.value;
		       if( /^ftp\./.test( hostStr )) {
			   hostStr = "ftp." + hostStr;
		       }
		       Components.classes["@mozilla.org/passwordmanager;1"]
			   .getService( Components.interfaces.nsIPasswordManager )
			   .removeUser( hostStr, this.userField.value );   
		   }
		   catch( e ) {
		       alert( e );
		   }
		   try {
		       // get accountsList
		       var accounts = bmhLib.RDF.GetResource( bmhLib.accountsNS + "accountList" );
		       bmhLib.RDFC.Init( bmhLib.prefsDS, accounts );
		       // remove account from list
		       bmhLib.RDFC.RemoveElement( accountRes, true );  
		   }
		   catch( e ) {
		       alert( e );
		   }
		   this.accountsMenu.menupopup.removeChild( this.accountsMenu.selectedItem );
		   var nrOfAccounts = this.accountsMenu.getElementsByTagName( "menuitem" ).length;
		   if( nrOfAccounts ) {
		       this.accountsMenu.selectedIndex = ( nrOfAccounts + index - 1 ) % nrOfAccounts;
		   }
		   else {
		       this.accountsMenu.selectedItem = null;
		   }
		   this.changeAccount();
	       },

newStylePrompt: function() {
		    if( this.saveStylePrompt( "", "" )) {
			this.stylesheetField.value = "";
			this.updateStyle( 3 );
		    }
		},

saveAccountAs: function( name, item ) {
		   if( ! item && this.accountsMenu.selectedItem ) {
		       // new account
		       this.hostField.value = "";
		       this.userField.value = "";
		       this.passwordField.value = "";
		       this.pathField.value = "";
		   }
		   // add account to datasource
		   var newAccount = bmhLib.RDF.GetResource( bmhLib.accountsNS + name );
		   var nameNode = bmhLib.RDF.GetLiteral( name );
		   var hostNode = bmhLib.RDF.GetLiteral( this.hostField.value );
		   var userNode = bmhLib.RDF.GetLiteral( this.userField.value );
		   var pathNode = bmhLib.RDF.GetLiteral( this.pathField.value );
		   var accounts = bmhLib.RDF.GetResource( bmhLib.accountsNS + "accountList" );
		   bmhLib.RDFC.Init( bmhLib.prefsDS, accounts );
		   // append new account to accounts list in styles datasource
		   bmhLib.RDFC.AppendElement( newAccount );  
		   bmhLib.prefsDS.Assert( newAccount, bmhLib.NameArc, nameNode, true );
		   bmhLib.prefsDS.Assert( newAccount, bmhLib.hostArc, hostNode, true );
		   bmhLib.prefsDS.Assert( newAccount, bmhLib.userArc, userNode, true );
		   bmhLib.prefsDS.Assert( newAccount, bmhLib.pathArc, pathNode, true );
		   // add password to password manager
		   try {
		       // if already there, no changes
		       var hostStr = this.hostField.value;
		       if( /^ftp\./.test( hostStr )) {
			   hostStr = "ftp." + hostStr;
		       }
		       Components.classes["@mozilla.org/passwordmanager;1"]
			   .getService( Components.interfaces.nsIPasswordManager )
			   .addUser( hostStr, this.userField.value, this.passwordField.value  );   
		   }
		   catch( e ) {}
		   var accountsMenuitem = document.createElement( "menuitem" );
		   accountsMenuitem.setAttribute( "label", name );
		   var val = encodeURI( this.hostField.value );
		   val += ( "|" + encodeURI( this.userField.value ));
		   val += ( "|" + encodeURI( this.passwordField.value ));
		   val += ( "|" + encodeURI( this.pathField.value ));
		   accountsMenuitem.value = val;
		   this.accountsMenu.menupopup.appendChild( accountsMenuitem );
		   this.accountsMenu.selectedItem = accountsMenuitem;
	       },

saveStyleAs: function( name, sheet ) { 
		 // create resource for new style
		 var newStyle = bmhLib.RDF.GetResource( bmhLib.stylesNS + name );
		 var nameNode = bmhLib.RDF.GetLiteral( name );
		 var sheetNode = bmhLib.RDF.GetLiteral( sheet );
		 var stylesList = bmhLib.RDF.GetResource( bmhLib.stylesNS + "stylesList" );
		 bmhLib.RDFC.Init( bmhLib.stylesDS, stylesList );
		 // append new style to styles list in styles datasource
		 bmhLib.RDFC.AppendElement( newStyle );  
		 // set properties in datasource of new style 
		 bmhLib.stylesDS.Assert( newStyle, bmhLib.nameArc, nameNode, true ); 
		 bmhLib.stylesDS.Assert( newStyle, bmhLib.sheetArc, sheetNode, true ); 
		 var insertIndex = this.stylesMenu.getElementsByTagName( "menuitem" ).length - 1;
		 var stylesMenuItem = this.stylesMenu.insertItemAt( insertIndex, name, sheet );
		 this.stylesheetField.value = sheet;
		 this.stylesMenu.selectedIndex = insertIndex;
	     },

deleteStylePrompt: function() {
		       var deletestyle = bmhLib.getLocaleString( "DeleteStyleString" );
		       var deleteconfirm = bmhLib.getLocaleString( "DeleteConfirmString" );
		       var name = this.getCurStyleName();
		       var promptService = this.getPromptService();
		       if( promptService.confirm( window, deletestyle, 
				   deleteconfirm + " " + name + "?" )) {
			   this.deleteStyle();
		       }
		   },

deleteStyle: function() {
		 var index = this.stylesMenu.selectedIndex;
		 var name = this.stylesMenu.selectedItem.label;
		 // name should be unique
		 var styleResource = bmhLib.RDF.GetResource( bmhLib.stylesNS + name );
		 var nameNode = bmhLib.stylesDS.GetTarget( styleResource, bmhLib.nameArc, true );
		 var sheetNode = bmhLib.stylesDS.GetTarget( styleResource, bmhLib.sheetArc, true );
		 // delete all properties of style from datasource 
		 bmhLib.stylesDS.Unassert( styleResource, bmhLib.nameArc, nameNode );
		 bmhLib.stylesDS.Unassert( styleResource, bmhLib.sheetArc, sheetNode );
		 // get stylesList
		 var stylesList = bmhLib.RDF.GetResource( bmhLib.stylesNS + "stylesList" );
		 bmhLib.RDFC.Init( bmhLib.stylesDS, stylesList );
		 // remove style from list
		 bmhLib.RDFC.RemoveElement( styleResource, true );  
		 // update options window
		 this.stylesMenu.removeItemAt( index );
		 var nrOfStyles = this.stylesMenu.getElementsByTagName( "menuitem" ).length;
		 this.stylesMenu.selectedIndex = ( nrOfStyles > 1 ? 0 : -1 ); 
		 this.changeStyle();
	     },

getOnlineStyles: function() {
		     if( bmhOptions.onlineStylesMenu.getElementsByTagName( "menuitem" ).length == 0 ) {
			 try {
			     var req = new XMLHttpRequest();
			     req.open( "GET", "http://bookmarkshome.mozdev.org/styles.html" );
			     req.onload = this.httpHtmlLoaded;
			     req.onerror = this.httpError;
			     req.setRequestHeader( "User-Agent", "Mozilla/5.0 (bmh)" );
			     req.overrideMimeType( "text/html" );
			     req.send( null );
			 }
			 catch( e ) {
			     alert( bmhLib.getLocale( "XMLHttpRequestFailureString" ));
			 }
		     }
		 },

httpCssLoaded: function( event ) {
		   var items = bmhOptions.stylesMenu.getElementsByAttribute( "label", 
			   bmhOptions.onlineLabel );
		   var onlineItem = ( items.length ? items[0] : null );
		   bmhOptions.stylesMenu.selectedItem = onlineItem; 
		   var sheet = bmhLib.formatSheet( event.target.responseText ); 
		   bmhOptions.onlineStylesMenu.selectedItem.value = sheet; 
		   bmhOptions.stylesheetField.value = sheet;
		   bmhOptions.updateStyle( 3 );
	       },

changeOnlineStyle: function( startLabel ) {
		       var items = bmhOptions.stylesMenu.getElementsByAttribute( "label", 
			       bmhOptions.onlineLabel );
		       var onlineItem = ( items.length ? items[0] : null );
		       bmhOptions.stylesMenu.selectedItem = onlineItem; 
		       var sheet = bmhOptions.onlineStylesMenu.selectedItem.value;
		       if( sheet ) {
			   bmhOptions.stylesheetField.value = sheet;
			   bmhOptions.updateStyle( 3 );
		       }
		       else {
			   var name = ( startLabel ? startLabel : 
				   bmhOptions.onlineStylesMenu.selectedItem.label );
			   var url = "http://bookmarkshome.mozdev.org/" + name + ".css";
			   try {
			       var req = new XMLHttpRequest();
			       req.open( "GET", url );
			       req.onload = this.httpCssLoaded;
			       req.onerror = this.httpError;
			       req.setRequestHeader( "User-Agent", "Mozilla/5.0 (bmh)" );
			       req.overrideMimeType( "text/css" );
			       req.send( null );
			   }
			   catch( e ) {
			       alert( bmhLib.getLocale( "XMLHttpRequestFailureString" ));
			   }

		       }
		   },

prevStyle: function() {
	       if( bmhOptions.onlineStylesMenu.selectedIndex > 0 ) {
		   bmhOptions.onlineStylesMenu.selectedIndex--;
		   bmhOptions.changeOnlineStyle();
	       }
	   },

nextStyle: function() {
	       var onlineLength = bmhOptions.onlineStylesMenu.getElementsByTagName( "menuitem" ).length;
	       if( bmhOptions.onlineStylesMenu.selectedIndex < onlineLength - 1 ) {
		   bmhOptions.onlineStylesMenu.selectedIndex++;
		   bmhOptions.changeOnlineStyle();
	       }
	   },

httpHtmlLoaded: function( event ) {
		    var page = event.target.responseText;
		    var selectToBottom = page.slice( page.search( /<select\s+id="styleSelect/ ));
		    var styleNames = selectToBottom.match( /<option[^<]+/g );
		    var matches;
		    for( var i = 0; i < styleNames.length; i++ ) {
			matches = styleNames[i].match( /<option[^>]+>([^<]+)$/ );
			if( matches.length ) {
			    styleNames[i] = matches[1];
			}
			bmhOptions.onlineStylesMenu.insertItemAt( i, styleNames[i], "" );
		    }
		    if( styleNames.length > 0 ) {
			bmhOptions.onlineStylesMenu.selectedIndex = 0;
			bmhOptions.onlineBox.style.display = "";
			bmhOptions.changeOnlineStyle( styleNames[0] );
		    }
		},

httpError: function( event ) {
	       event.target.abort();
	       alert( event );
	   },

getCurStyleName: function() {
		     var selectedLabel = this.stylesMenu.selectedItem.label;
		     if( selectedLabel == this.onlineLabel ) { 
			 return bmhOptions.onlineStylesMenu.selectedItem.label;
		     }
		     else {
			 return selectedLabel;
		     }
		 },

getNrOfStyleItems: function() {
		       return this.stylesMenu.getElementsByTagName( "menuitem" ).length;
		   },

showColor: function( color ) {
	       document.getElementById( "hexColor" ).value = color;
	   },

changeStyle: function() {
		 var selItem = this.stylesMenu.selectedItem;
		 if( ! selItem ) {
		     return;
		 }
		 if( selItem.label == this.onlineLabel ) {
		     this.getOnlineStyles();
		 }
		 else {
		     this.stylesheetField.value = selItem.value;
		     this.updateStyle( 3 );
		 }
	     },

saveFile: function() {
	      try {
		  var nsIFilePicker = Components.interfaces.nsIFilePicker;
		  var fp = Components.classes["@mozilla.org/filepicker;1"]
		      .createInstance( nsIFilePicker );
		  // just let fp determine the folder
		  /*
		     var  homeURL = Components.classes["@mozilla.org/file/directory_service;1"] 
		     .getService( Components.interfaces.nsIProperties )
		     .get( "Home", Components.interfaces.nsIFile )
		     .QueryInterface( Components.interfaces.nsILocalFile )
		     .path; 
		     var homeDir = Components.classes["@mozilla.org/file/local;1"]
		     .createInstance( Components.interfaces.nsILocalFile );
		     homeDir.initWithPath( homeURL );
		  // set home as displayed directory
		  fp.displayDirectory = homeDir; 
		  var dfltStr = bmhLib.getWhat( this.optionsURL ) + "Saved.html"; 
		  homeDir.append( dfltStr );
		  // nsIFile.leafName returns value of type AString and can be 
		  // used to initialize nsIFilePicker.defaultString
		  fp.defaultString = homeDir.leafName;
		   */
		  // for some reason I thought assigning the string didn't work
		  var dfltStr = bmhLib.getWhat( this.optionsURL ) + "Saved.html"; 
		  fp.defaultString = dfltStr;
		  fp.appendFilters( nsIFilePicker.filterHTML );
		  fp.init( window, bmhLib.getLocaleString( "SavePageAsString" ), nsIFilePicker.modeSave );
		  var action, outFile;
		  do {
		      action = fp.show();
		      if( action == 1 ) {
			  // cancel
			  return;
		      }
		      outFile = fp.file;
		      if( ! /\.html{0,1}$/.test( fp.file.path )) {
			  outFile.initWithPath( fp.file.path + ".html" );
		      }
		      fp.defaultString = outFile.leafName;
		  }
		  // after adding 'html', file may exist
		  while( outFile.exists() && ( action == 0 ));
		  var staticFile = bmhLib.getBmhFile( this.optionsURL );
		  /*
		     // doesn't work when outFile exists
		  transportService = Components.classes["@mozilla.org/network/stream-transport-service;1"]
		      .getService( Components.interfaces.nsIStreamTransportService );
		  var transport = transportService.createOutputTransport( fileOutStream, 0, -1, true );
		  var outStream = transport.openOutputStream( 0, 0, 0 );
		  var fileInStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
		      .createInstance( Components.interfaces.nsIFileInputStream );
		  fileInStream.init( staticFile, 0x01, 0644, 0 );
		  outStream.writeFrom( fileInStream, staticFile.fileSize );
		  */
		  // read
		  var fileInStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
		      .createInstance( Components.interfaces.nsIFileInputStream );
		  fileInStream.init( staticFile, 0x01, 0644, 0 );
		  var scriptInStream = Components.classes["@mozilla.org/scriptableinputstream;1"].
		      createInstance( Components.interfaces.nsIScriptableInputStream );
		  scriptInStream.init( fileInStream );
		  var doc = scriptInStream.read( staticFile.fileSize );
		  // write
		  var fileOutStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
		      .createInstance( Components.interfaces.nsIFileOutputStream );
		  fileOutStream.init( outFile, 0x20|0x02|0x08, 0644, 0 ); 
		  fileOutStream.write( doc, staticFile.fileSize );
		  fileOutStream.close();
		  /*
		     // doesn't work when outFile exists
		     var url = fp.fileURL;
		     var path = url.directory;
		     var fileName = url.fileName;
		     var saveDir = Components.classes["@mozilla.org/file/local;1"]
		     .createInstance( Components.interfaces.nsILocalFile );
		     saveDir.initWithPath( outFile.parent.path ); 
		     staticFile.copyTo( saveDir, outFile.leafName );
		   */
	      }
	      catch( e ) {
		  alert( bmhLib.getLocaleString( "SavePageFailureString", [this.optionsURL] ));
		  throw( e );
	      }
	  },

parseBookmarks: function() {
		    var curName = bmhLib.getProperty( bmhLib.bmRoot, bmhLib.NameArc, bmhLib.BMDS );
		    var curType;
		    var curFolder = { node: bmhLib.bmRoot, ancestors: [curName] };
		    var folderStack = new Array( curFolder );
		    var enumerator, tempFolders, curNode; 
		    var curNodeAncestors, isChecked, isExcluded, i;
		    while( folderStack.length > 0 ) {
			curFolder = folderStack.pop();
			bmhLib.RDFC.Init( bmhLib.BMDS, curFolder.node );
			enumerator = bmhLib.RDFC.GetElements();
			tempFolders = new Array();
			while( enumerator.hasMoreElements() ) {
			    curNode = enumerator.getNext();
			    curType = bmhLib.getProperty( curNode, bmhLib.typeArc, bmhLib.BMDS )  
				.split( "#" )[1]; 
			    if( curType == "Folder" || curType == "Livemark" ) {
				curName = bmhLib.getProperty( curNode, bmhLib.NameArc, bmhLib.BMDS );
				curNodeAncestors = curFolder.ancestors.concat( [curName] );
				tempFolders.unshift( { node: curNode, ancestors: curNodeAncestors } );
			    }
			}
			this.createFolder( curFolder.ancestors );  
			folderStack = folderStack.concat( tempFolders );
		    }
		},

createFolder: function( ancestors ) {  
		  // hbox for checkbox and indentation spacer
		  var folderBox = this.contentFrame.document.createElement( "hbox" );
		  var twisty = document.createElement( "button" );
		  twisty.setAttribute( "type", "disclosure" );
		  twisty.setAttribute( "id", "twt" + this.folderBoxes.length );
		  folderBox.appendChild( twisty );
		  // checkbox
		  var folderCheckbox = this.contentFrame.document.createElement( "checkbox" );
		  folderCheckbox.setAttribute( "id", "fcb" + this.folderBoxes.length );
		  folderBox.appendChild( folderCheckbox );
		  // add checkbox to array
		  var folderObj = { value: ancestors, checked: false }
		  this.folders.push( folderObj );
		  this.sortedFolders.push( folderObj ); 
		  this.folderBoxes.push( folderBox );
	      },

sortFolders: function( button ) {
		 var sortLabel = bmhLib.getLocaleString( "SortLabel" );
		 var unsortLabel = bmhLib.getLocaleString( "UnsortLabel" );
		 if( button.label ==  sortLabel ) {
		     button.setAttribute( "label", unsortLabel ); 
		     this.drawFolders( this.sortedFolders );
		 }
		 else {
		     button.setAttribute( "label", sortLabel );
		     this.drawFolders( this.folders );
		 }
	     },

drawFolders: function( folders ) {
		 var curVal = folders[0].value;
		 var curVbox = this.inOrExcludedFoldersBox;
		 if( curVbox.lastChild ) {
		     // remove checkboxes, then add them in different order
		     curVbox.removeChild( curVbox.lastChild );
		 }
		 else {
		     var checkbox = this.folderBoxes[0].lastChild;
		     checkbox.setAttribute( "label", curVal[curVal.length-1] );
		     checkbox.setAttribute( "checked", folders[0].checked );
		     curVbox.appendChild( this.folderBoxes[0] );
		 }
		 var folderVbox, twisty; 
		 for( var i = 1; i < folders.length; i++ ) {
		     curVal = folders[i].value;
		     twisty = this.folderBoxes[i-1].firstChild; 
		     if( curVal.length > folders[i-1].value.length ) {
			 folderVbox = document.createElement( "vbox" );
			 folderVbox.setAttribute( "style", "padding-left: 30px" );
			 curVbox.appendChild( folderVbox );
			 curVbox = folderVbox;
			 twisty.style.visibility = "";
			 twisty.open = true;
		     }
		     else {
			 if( curVal.length < folders[i-1].value.length ) {
			     curVbox = curVbox.parentNode;
			 }
			 twisty.style.visibility = "hidden";
		     }
		     checkbox = this.folderBoxes[i].lastChild;
		     checkbox.setAttribute( "label", curVal[curVal.length-1] );
		     checkbox.setAttribute( "checked", folders[i].checked );
		     curVbox.appendChild( this.folderBoxes[i] );
		 }
		 twisty = this.folderBoxes[i-1].firstChild; 
		 twisty.style.visibility = "hidden";
	     },

checkFolders: function() {
		  var excludePref = this.excludeMenu.selectedIndex;
		  var exclude = ( excludePref == 0 ? true : false );
		  var inOrExcludedFolders = bmhLib.getArrayFromRDFPref( "inOrExcludedFolders", this.optionsURL );
		  var isChecked;
		  for( var i = 0; i < this.folders.length; i++ ) {
		      isExcluded = bmhLib.isExcludedFolder( this.folders[i].value,
			      excludePref, inOrExcludedFolders );
		      isChecked = ( exclude == isExcluded ? true : false );
		      this.folders[i].checked = isChecked;
		  }
	      },

checkSubfolders: function( node ) {
		     if( node.nodeName == "button" ) {
			 node.open = ( node.open ? false : true );
			 var display = ( node.open ? "" : "none" );
			 node.parentNode.nextSibling.style.display = display;
		     }
		     else {
			 var sortLabel = this.sortButton.label;
			 sortProp = bmhLib.getLocaleString( "SortLabel" );
			 var folders = ( sortLabel == sortProp ? this.folders : this.sortedFolders );
			 // id is "fcb" + index
			 var startIndex = parseInt( node.id.substr( 3 ));
			 folders[startIndex].checked = node.checked;
			 if( document.getElementById( "subfolderCheckbox" ).checked ) {
			     var l = folders[startIndex].value.length;
			     var i = startIndex + 1;
			     while( i < folders.length && ( folders[i].value.length > l )) {
				 this.folderBoxes[i].lastChild.checked = node.checked;
				 folders[i].checked = node.checked;
				 i++;
			     }
			 }
		     }
		 }
}

