/**
 * aboutBookmarkshomeModules.js
 * JavaScript framework for JavaScript-based XPCOM component registration
 * Mook 2004 / 2005 adapted by Jeroen Groenenboom
 * License: MPL
 */

// all XPCOM components this can make (all fields are required)
var comp = {
Constructor: null,	// the name of the constructor (as a string)
ProgID: null,		// the contract-id of the component
CID: null,			// the component id of the component
// made via Components.ID(GUID-as-a-string)
Description: null	// a description of the component (never used?)
}

var KnownComponents = [ 
// five bmh pages
{ Constructor: "aboutBookmarkshome", 
ProgID: "@mozilla.org/network/protocol/about;1?what=bookmarkshome",
CID: Components.ID( "{2d48bac3-b9c6-4db4-9b96-0d05754ec3c8}" ),
Description: "about:bookmarkshome handler" },

{ Constructor: "aboutBookmarkshome", 
ProgID: "@mozilla.org/network/protocol/about;1?what=bmh1",
CID: Components.ID( "{cf953e12-2205-4c8a-8d27-12f4f97bbe59}" ),
Description: "about:bmh1 handler" },

{ Constructor: "aboutBookmarkshome", 
ProgID: "@mozilla.org/network/protocol/about;1?what=bmh2",
CID: Components.ID( "{6bfaa945-4ad8-4923-86c5-28a11bb5edfa}" ),
Description: "about:bmh2 handler" },

{ Constructor: "aboutBookmarkshome", 
ProgID: "@mozilla.org/network/protocol/about;1?what=bmh3",
CID: Components.ID( "{8fec5e86-8176-43fb-9c89-7c6206162718}" ),
Description: "about:bmh3 handler" },

{ Constructor: "aboutBookmarkshome", 
ProgID: "@mozilla.org/network/protocol/about;1?what=bmh4",
CID: Components.ID( "{a029bd5e-e153-419a-a363-518083d6b007}" ),
Description: "about:bmh4 handler" },
    
    // bookmarks observer
{ Constructor: "bmhBookmarksObserverService", 
ProgID: "@jeroen/bmhBookmarksObserverService;1",
CID: Components.ID( "{09a1eefc-bd3c-4dcc-990f-686067d713fb}" ),
Description: "bookmarks observer" }
];

// the list of JavaScript files placed in the same directory to read from
// this is used so we can implement the XPCOM components in external files
var ExternalScripts = ["aboutBookmarkshome.js","bmhBookmarksObserverService.js"];

// the name of the module (used only as extra info on component registration)
var ModuleName = "aboutBookmarkshomeModule";

////////// IMPLEMENTATION DETAILS BELOW //////////

/**
 * This loads external scripts at run time, in the same directory as this file
 */
(function(){
 var selfname = Components.Exception().filename;	// get the name of this file
 var fileProtocolHandler =
 Components.classes['@mozilla.org/network/protocol;1?name=file']
 .createInstance( Components.interfaces['nsIFileProtocolHandler'] );
 var bmhSelf = fileProtocolHandler.getFileFromURLSpec( selfname );
 var bmhParent = bmhSelf.parent; // the directory this file is in
 var loader = Components.classes['@mozilla.org/moz/jssubscript-loader;1']
 .createInstance( Components.interfaces['mozIJSSubScriptLoader'] );

 for ( var index in ExternalScripts ) {
 try {
 var child = bmhParent.clone();
 child.append( ExternalScripts[index] );
 // child is a nsILocalFile of the file we want to load
 var childname = fileProtocolHandler.getURLSpecFromFile( child );
 loader.loadSubScript( childname );
 } catch (ex) {
 dump( 'JavaScript-XPCOM module error:\n' );
 dump( 'While loading external script "' 
     + ExternalScripts[index] + '":\n' );
 dump( ex + '\n' );
 try {
     dump( 'File: ' + ex.filename + '\n' );
     dump( 'Line: ' + ex.lineNumber + '\n' );
     if ( ex.location ) {
	 dump( 'Stack File: ' + ex.location.filename + '\n' );
	 dump( 'Stack Line: ' + ex.location.lineNumber + '\n' );
	 dump( 'Stack Source: ' + ex.location.sourceLine + '\n' );
     }
 } catch ( e ) {
     // we had problems dumping the error - too bad
     dump( '(Error retrieving error location)\n' );
 }
 throw( ex );
 }
 }
})();

/**
 * This is the module handler for all JavaScript based XPCOM modules.
 */
var mihoJSModules = {
_firstTime: true,

	    registerSelf: function ( aComponentManager, aFileSpec, aLocation, aType ) {
		var index, info;
		if ( this._firstTime ) {
		    this._firstTime = false;
		    throw Components.results['NS_ERROR_FACTORY_REGISTER_AGAIN'];
		}
		aComponentManager = aComponentManager
		    .QueryInterface( Components.interfaces['nsIComponentRegistrar'] );

		// No idea how to do this only for debug builds... :p
		dump( "*** Registering " + ModuleName
			+ " components (all right -- a JavaScript module!)\n");

		//initComponents();
		for ( index in KnownComponents ) {
		    var info = KnownComponents[index];
		    if ("undefined" == typeof( info.CID ) || ! ( info.CID ))
			continue;
		    aComponentManager.registerFactoryLocation(
			    info.CID,
			    info.Description,
			    info.ProgID,
			    aFileSpec,
			    aLocation,
			    aType
			    );
		}
	    },

getClassObject: function ( aComponentManager, aCID, aIID ) {
		    var index, info;
		    for ( index in KnownComponents ) {
			info = KnownComponents[index];
			if (( "undefined" != typeof( info.CID )) && ( info.CID ) 
				&& ( aCID.equals(info.CID )))
			    return {
createInstance: function( aOuter, aIID ) {
		    if ( aOuter != null )
			throw Components.results['NS_ERROR_NO_AGGREGATION'];
		    return ( new ( eval( info.Constructor )))
			.QueryInterface( aIID );
		},
lockFactory: function ( aLock ) {
		 throw Components.results['NS_ERROR_NOT_IMPLEMENTED'];
	     },
QueryInterface: function( aIID ) {
		    if ( aIID.equals( Components.interfaces['nsISupports'] )
			    || aIID.equals( Components.interfaces['nsIFactory'] ))
			return this;
		    throw Components.results['NS_ERROR_NO_INTERFACE'];
		}
			    };
		    }

		    if (!aIID.equals( Components.interfaces['nsIFactory'] ))
			throw Components.results['NS_ERROR_NOT_IMPLEMENTED'];

		    throw Components.results['NS_ERROR_NO_INTERFACE'];
		},

canUnload: function ( aComponentManager ) {
	       return true;
	   }
};

function NSGetModule( aComponentManager, aFileSpec ) {
    return mihoJSModules;
}
